//   Copyright (c) 2012 Anna M. Bigatti, Bruno Simoes
//
//   This file is part of the source of CoCoALib, the CoCoA Library.
//
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include "BuiltInFunctions.H"
#include "BuiltInOneLiners.H"

using namespace std;
using namespace boost;
using namespace boost::iostreams;
using namespace CoCoA::AST;
using namespace CoCoA::LexerNS;
using namespace CoCoA::ParserNS;

namespace CoCoA {
namespace InterpreterNS {

//  extern std::vector<NameFunPair> builtIns; // declared in BuiltInFunctions.C

#ifdef CoCoA_WITH_GSL

  //---- one-liners
  DECLARE_COCOALIB_FUNCTION1(LU, MAT)
  DECLARE_COCOALIB_FUNCTION1(CholeskyDecomposition, MAT)
  DECLARE_COCOALIB_FUNCTION1(SVD, MAT)
  DECLARE_COCOALIB_FUNCTION1(SingularValues, MAT)
  DECLARE_COCOALIB_FUNCTION1(QR, MAT)
  DECLARE_COCOALIB_FUNCTION1(QRPT, MAT)
 

  
  //---- other functions
  DECLARE_STD_BUILTIN_FUNCTION(SolverLU, 2) {
    intrusive_ptr<MAT> A = runtimeEnv->evalArgAs<MAT>(ARG(0));
    vector<RingElem> b = runtimeEnv->evalArgAsListOfRingElem(ARG(1));
    vector<RingElem> x = SolverLU(A->theMatrix, b);
    return new LIST(x);
  }
  END_STD_BUILTIN_FUNCTION


  DECLARE_STD_BUILTIN_FUNCTION(SolverSVD, 2) {
    intrusive_ptr<MAT> A = runtimeEnv->evalArgAs<MAT>(ARG(0));
    vector<RingElem> b = runtimeEnv->evalArgAsListOfRingElem(ARG(1));//, R->theRing);
    vector<RingElem> x = SolveSVD(A->theMatrix, b);
    return new LIST(x);
  }
  END_STD_BUILTIN_FUNCTION


  DECLARE_STD_BUILTIN_FUNCTION(SolverQR, 2) {
    intrusive_ptr<MAT> A = runtimeEnv->evalArgAs<MAT>(ARG(0));
    vector<RingElem> b = runtimeEnv->evalArgAsListOfRingElem(ARG(1));
    vector<RingElem> x = SolverQR(A->theMatrix, b);
    return new LIST(x);
  }
  END_STD_BUILTIN_FUNCTION


  DECLARE_STD_BUILTIN_FUNCTION(BidiagDecomposition, 4) {
    intrusive_ptr<MAT> A = runtimeEnv->evalArgAs<MAT>(ARG(0));
    intrusive_ptr<MAT> D = runtimeEnv->evalArgAs<MAT>(ARG(1));
    vector<RingElem> tau_U = runtimeEnv->evalArgAsListOf<RINGELEM>(ARG(2));
    vector<RingElem> tau_V = runtimeEnv->evalArgAsListOf<RINGELEM>(ARG(3)); 
    BidiagDecomposition(A->theMatrix, D->theMatrix, tau_U, tau_V);
    return VoidValue::theInstance;
  }
  END_STD_BUILTIN_FUNCTION


  DECLARE_STD_BUILTIN_FUNCTION(BidiagDecompUnpack, 5) {
    intrusive_ptr<MAT> A = runtimeEnv->evalArgAs<MAT>(ARG(0));
    intrusive_ptr<MAT> U = runtimeEnv->evalArgAs<MAT>(ARG(1));
    intrusive_ptr<MAT> V = runtimeEnv->evalArgAs<MAT>(ARG(2));
    vector<RingElem> tau_U = runtimeEnv->evalArgAsListOf<RINGELEM>(ARG(3));
    vector<RingElem> tau_V = runtimeEnv->evalArgAsListOf<RINGELEM>(ARG(4)); 
    BidiagDecompUnpack(A->theMatrix, U->theMatrix, V->theMatrix, tau_U, tau_V);
    return VoidValue::theInstance;
  }
  END_STD_BUILTIN_FUNCTION


#endif //  CoCoA_WITH_GSL

} // namespace AST
} // namespace CoCoA
